JOB_TYPE_FORECAST_SERIES = "job_forecast"
JOB_TYPE_DETECT_ANOMALIES_FOR_SERIES = "job_anomaly_detect"
JOB_TYPE_BASE_SERIES_ANALYSIS = "job_base_analysis"
JOB_TYPE_STATIC_RULES = "job_static_rules"

JOB_TYPES = [
    JOB_TYPE_BASE_SERIES_ANALYSIS,
    JOB_TYPE_FORECAST_SERIES,
    JOB_TYPE_DETECT_ANOMALIES_FOR_SERIES,
    JOB_TYPE_STATIC_RULES
]

JOB_STATUS_NONE = None
JOB_STATUS_OPEN = 1
JOB_STATUS_PENDING = 2
JOB_STATUS_DONE = 3
JOB_STATUS_FAILED = 4
